﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.Storage;
using Windows.Storage.Streams;

namespace AnomalyDetection.Storage
{
    public class TemperatureFileStorage
    {
        public string FilePath
        {
            get { return workingFile.Path; }
        }

        private const string fileName = "SenseHatTemperatureData.csv";
        private const string columnName = "Temperature";

        private string folderName = Package.Current.DisplayName;

        private StorageFolder workingFolder;
        private StorageFile workingFile;

        public static async Task<TemperatureFileStorage> CreateAsync()
        {
            var temperatureFileStorage = new TemperatureFileStorage();

            await temperatureFileStorage.PrepareFolder();
            await temperatureFileStorage.PrepareFile();

            return temperatureFileStorage;
        }

        public async Task WriteData(List<float> temperatureDataset)
        {
            var randomAccessStream = await workingFile.OpenAsync(FileAccessMode.ReadWrite);

            using (var dataWriter = new DataWriter(randomAccessStream))
            {
                WriteLine(dataWriter, columnName);

                foreach (float temperature in temperatureDataset)
                {
                    WriteLine(dataWriter, temperature.ToString());
                }

                await dataWriter.StoreAsync();
            }
        }

        // Create an empty constructor, so it's not publicly accessible
        private TemperatureFileStorage() { }

        private async Task PrepareFolder()
        {
            var storageFolder = ApplicationData.Current.TemporaryFolder;

            // Check if folder already exists
            var storageItem = await storageFolder.TryGetItemAsync(folderName);

            if (storageItem == null)
            {
                // ... if not, create one
                storageFolder = await storageFolder.CreateFolderAsync(folderName);
            }
            else
            {
                storageFolder = (StorageFolder)storageItem;
            }

            workingFolder = storageFolder;
        }

        private async Task PrepareFile()
        {
            // Create file, overwriting the previous one
            workingFile = await workingFolder.CreateFileAsync(fileName,
                CreationCollisionOption.ReplaceExisting);
        }

        private void WriteLine(DataWriter dataWriter, string value)
        {
            dataWriter.WriteString(value);
            dataWriter.WriteString("\r\n");
        }

        private void WriteLine(DataWriter dataWriter, string value1, float value2)
        {
            dataWriter.WriteString(value1);
            dataWriter.WriteString(",");
            dataWriter.WriteString(value2.ToString());
            dataWriter.WriteString("\r\n");
        }        
    }
}
