﻿using Microsoft.ProjectOxford.Emotion.Contract;
using SenseHat.Helpers;
using System;
using System.Linq;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Shapes;

namespace EmotionsDetector.Helpers
{
    public static class EmotionHelper
    {
        public static string GetTopEmotionName(Emotion emotion)
        {
            Check.IsNull(emotion);

            var rankedList = emotion.Scores.ToRankedList();

            return rankedList.First().Key;
        }

        public static Color GetEmotionColor(string emotionName)
        {
            Check.IsNull(emotionName);

            // Dummy object for reading emotion names
            var scores = new Scores();

            switch (emotionName)
            {
                case nameof(scores.Happiness):
                    return Colors.GreenYellow;

                case nameof(scores.Anger):
                    return Colors.Red;

                case nameof(scores.Disgust):
                    return Colors.OrangeRed;

                case nameof(scores.Surprise):
                    return Colors.HotPink;

                case nameof(scores.Sadness):
                    return Colors.Brown;

                case nameof(scores.Fear):
                    return Colors.Gray;

                case nameof(scores.Contempt):
                    return Colors.Purple;

                default:
                case nameof(scores.Neutral):
                    return Colors.White;
            }
        }

        public static TextBlock PrepareEmotionTextBlock(Rectangle faceBox, Color emotionColor, string emotionName)
        {
            Check.IsNull(faceBox);
            Check.IsNull(emotionColor);
            Check.IsNull(emotionName);

            var textBlock = new TextBlock()
            {
                Foreground = new SolidColorBrush(emotionColor),
                FontSize = 38,
                Text = emotionName
            };

            // Measure text block
            textBlock.Measure(Size.Empty);

            // Calculate offsets
            var xTextBlockOffset = (faceBox.ActualWidth - textBlock.ActualWidth) / 2.0;
            var yTextBlockOffset = -textBlock.ActualHeight;

            // Ignore negative horizontal offset
            xTextBlockOffset = Math.Max(0, xTextBlockOffset);

            // Translate text block, so it is centered with respect to face box
            var faceBoxTranslateTransform = faceBox.RenderTransform as TranslateTransform;

            textBlock.RenderTransform = new TranslateTransform()
            {
                X = faceBoxTranslateTransform.X + xTextBlockOffset,
                Y = faceBoxTranslateTransform.Y + yTextBlockOffset
            };

            return textBlock;
        }

        public static Rectangle PrepareFaceBox(Microsoft.ProjectOxford.Common.Rectangle faceRectangle, Color emotionColor, double xScalingFactor, double yScalingFactor)
        {
            // Prepare bounding rectangle
            var faceBox = new Rectangle()
            {
                Stroke = new SolidColorBrush(emotionColor),
                StrokeThickness = 5,
                Width = faceRectangle.Width * xScalingFactor,
                Height = faceRectangle.Height * yScalingFactor
            };

            // Translate bounding rectangle
            var translateTransform = new TranslateTransform()
            {
                X = faceRectangle.Left * xScalingFactor,
                Y = faceRectangle.Top * yScalingFactor
            };

            faceBox.RenderTransform = translateTransform;

            return faceBox;
        }
    }
}
