﻿using CustomIoTSolution.Common.Config;
using CustomIoTSolution.Common.Models;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Table;
using SenseHat.Portable.Helpers;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace CustomIoTSolution.Common.Helpers
{
    public static class AzureStorageHelper
    {
        private static CloudTableClient cloudTableClient;

        private static EntityResolver<DynamicTableEntity> dynamicTableEntityResolver = 
            (partitionKey, rowKey, timestamp, properties, etag) => 
        {
            return new DynamicTableEntity(partitionKey, rowKey, etag, properties);
        };

        static AzureStorageHelper()
        {
            ConfigureTableClient();
        }

        public static async Task<DynamicTableEntity> GetMostRecentEntity(string tableName, DateTimeOffset dateTimeOffset)
        {
            // Verify input arguments
            Check.IsNull(tableName);
            Check.IsNull(dateTimeOffset);

            // Get cloud table client
            var cloudTable = cloudTableClient.GetTableReference(tableName);

            // Create timestamp filter
            var timestampFilter = TableQuery.GenerateFilterConditionForDate("Timestamp", "gt", dateTimeOffset);

            // Crete and execute query
            var tableQuery = new TableQuery()
            {
                FilterString = timestampFilter.ToString()
            };

            // Execute query and filter to get the most recent entity
            return (await cloudTable.ExecuteQuerySegmentedAsync(tableQuery, dynamicTableEntityResolver, null))
                .OrderByDescending(x => x.Timestamp)
                .FirstOrDefault();
        }

        public static async Task WriteAlertToAzureTable(AlertEntity alertEntity)
        {
            // Verify input arguments
            Check.IsNull(alertEntity);

            // Get table reference
            var cloudTable = cloudTableClient.GetTableReference(Configuration.AlertsTableName);

            // Configure table operation to insert AlertEntity
            var tableOperation = TableOperation.Insert(alertEntity);

            // Execute request
            await cloudTable.ExecuteAsync(tableOperation);
        }

        private static void ConfigureTableClient()
        {
            var cloudStorageAccount = new CloudStorageAccount(Configuration.GetStorageCredentials(), true);

            cloudTableClient = cloudStorageAccount.CreateCloudTableClient();
        }
    }
}
