﻿using Microsoft.Azure.Devices;
using System.Threading.Tasks;
using Windows.Storage;

namespace CustomIoTSolution.RemoteClient.Helpers
{
    public static class DeviceRegistrationHelper
    {
        private static string DeviceKeyString = "DeviceKey";

        public static async Task<Device> RegisterDevice(string connectionString, string deviceId)
        {
            // Associate connection with the IoT Hub
            var registryManager = RegistryManager.CreateFromConnectionString(connectionString);

            // Check if device already exists?
            var device = await registryManager.GetDeviceAsync(deviceId);

            if (device == null)
            {
                // If not, register a device
                device = await registryManager.AddDeviceAsync(new Device(deviceId));

                // ... and store device primary key in the local settings
                StoreDeviceKey(device.Authentication.SymmetricKey.PrimaryKey);
            }

            return device;
        }

        public static async Task RemoveDevice(string connectionString, string deviceId)
        {
            // Associate connection with the IoT Hub
            var registryManager = RegistryManager.CreateFromConnectionString(connectionString);

            // Check if device exists
            Device device = await registryManager.GetDeviceAsync(deviceId);
            if (device != null)
            {
                // If so, remove a device
                await registryManager.RemoveDeviceAsync(deviceId);

                // Clear device key in the app settings
                StoreDeviceKey(null);
            }
        }

        public static string RetrieveDeviceKey()
        {
            var localSettings = ApplicationData.Current.LocalSettings;

            return localSettings.Values[DeviceKeyString] as string;
        }        

        private static void StoreDeviceKey(string deviceKey)
        {
            var localSettings = ApplicationData.Current.LocalSettings;

            localSettings.Values[DeviceKeyString] = deviceKey;
        }                
    }
}
